/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.app.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import javax.validation.Valid;

import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.app.entity.PaymentAccountApplyment;
import org.springblade.modules.app.vo.PaymentAccountApplymentVO;
import org.springblade.modules.app.service.IPaymentAccountApplymentService;
import org.springblade.core.boot.ctrl.BladeController;

/**
 * 商户进件 控制器
 *
 * @author BladeX
 * @since 2022-10-14
 */
@RestController
@AllArgsConstructor
@RequestMapping("app/paymentaccountapplyment")
@Api(value = "商户进件", tags = "商户进件接口")
public class PaymentAccountApplymentController extends BladeController {

	private final IPaymentAccountApplymentService paymentAccountApplymentService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	//@ApiOperationSupport(order = 1)
	//@ApiOperation(value = "详情", notes = "传入paymentAccountApplyment")
	public R<PaymentAccountApplyment> detail(PaymentAccountApplyment paymentAccountApplyment) {
		PaymentAccountApplyment detail = paymentAccountApplymentService.getOne(Condition.getQueryWrapper(paymentAccountApplyment));
		return R.data(detail);
	}

	/**
	 * 分页 商户进件
	 */
	@GetMapping("/list")
	//@ApiOperationSupport(order = 2)
	//@ApiOperation(value = "分页", notes = "传入paymentAccountApplyment")
	public R<IPage<PaymentAccountApplyment>> list(PaymentAccountApplyment paymentAccountApplyment, Query query) {
		IPage<PaymentAccountApplyment> pages = paymentAccountApplymentService.page(Condition.getPage(query), Condition.getQueryWrapper(paymentAccountApplyment));
		return R.data(pages);
	}

	/**
	 * 自定义分页 商户进件
	 */
	@GetMapping("/page")
	//@ApiOperationSupport(order = 3)
	//@ApiOperation(value = "分页", notes = "传入paymentAccountApplyment")
	public R<IPage<PaymentAccountApplymentVO>> page(PaymentAccountApplymentVO paymentAccountApplyment, Query query) {
		IPage<PaymentAccountApplymentVO> pages = paymentAccountApplymentService.selectPaymentAccountApplymentPage(Condition.getPage(query), paymentAccountApplyment);
		return R.data(pages);
	}

	/**
	 * 新增 商户进件
	 */
	@PostMapping("/save")
	//@ApiOperationSupport(order = 4)
	//@ApiOperation(value = "新增", notes = "传入paymentAccountApplyment")
	public R save(@Valid @RequestBody PaymentAccountApplyment paymentAccountApplyment) {
		return R.status(paymentAccountApplymentService.save(paymentAccountApplyment));
	}

	/**
	 * 修改 商户进件
	 */
	@PostMapping("/update")
	//@ApiOperationSupport(order = 5)
	//@ApiOperation(value = "修改", notes = "传入paymentAccountApplyment")
	public R update(@Valid @RequestBody PaymentAccountApplyment paymentAccountApplyment) {
		return R.status(paymentAccountApplymentService.updateById(paymentAccountApplyment));
	}

	/**
	 * 新增或修改 商户进件
	 */
	@PostMapping("/submit")
	//@ApiOperationSupport(order = 6)
	//@ApiOperation(value = "新增或修改", notes = "传入paymentAccountApplyment")
	public R submit(@Valid @RequestBody PaymentAccountApplyment paymentAccountApplyment) {
		return R.status(paymentAccountApplymentService.saveOrUpdate(paymentAccountApplyment));
	}

	
	/**
	 * 删除 商户进件
	 */
	@PostMapping("/remove")
	//@ApiOperationSupport(order = 8)
	//@ApiOperation(value = "删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(paymentAccountApplymentService.removeByIds(Func.toLongList(ids)));
	}

	
}
