/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.app.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import javax.validation.Valid;

import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.modules.app.entity.PaymentOrderRecord;
import org.springblade.modules.app.vo.PaymentOrderRecordVO;
import org.springblade.modules.app.service.IPaymentOrderRecordService;
import org.springblade.core.boot.ctrl.BladeController;

/**
 * 支付明细表 控制器
 *
 * @author BladeX
 * @since 2022-10-14
 */
@RestController
@AllArgsConstructor
@RequestMapping("app/paymentorderrecord")
@Api(value = "支付明细表", tags = "支付明细表接口")
public class PaymentOrderRecordController extends BladeController {

	private final IPaymentOrderRecordService paymentOrderRecordService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	//@ApiOperationSupport(order = 1)
	//@ApiOperation(value = "详情", notes = "传入paymentOrderRecord")
	public R<PaymentOrderRecord> detail(PaymentOrderRecord paymentOrderRecord) {
		PaymentOrderRecord detail = paymentOrderRecordService.getOne(Condition.getQueryWrapper(paymentOrderRecord));
		return R.data(detail);
	}

	/**
	 * 分页 支付明细表
	 */
	@GetMapping("/list")
	//@ApiOperationSupport(order = 2)
	//@ApiOperation(value = "分页", notes = "传入paymentOrderRecord")
	public R<IPage<PaymentOrderRecord>> list(PaymentOrderRecord paymentOrderRecord, Query query) {
		IPage<PaymentOrderRecord> pages = paymentOrderRecordService.page(Condition.getPage(query), Condition.getQueryWrapper(paymentOrderRecord));
		return R.data(pages);
	}

	/**
	 * 自定义分页 支付明细表
	 */
	@GetMapping("/page")
	//@ApiOperationSupport(order = 3)
	//@ApiOperation(value = "分页", notes = "传入paymentOrderRecord")
	public R<IPage<PaymentOrderRecordVO>> page(PaymentOrderRecordVO paymentOrderRecord, Query query) {
		IPage<PaymentOrderRecordVO> pages = paymentOrderRecordService.selectPaymentOrderRecordPage(Condition.getPage(query), paymentOrderRecord);
		return R.data(pages);
	}

	/**
	 * 新增 支付明细表
	 */
	@PostMapping("/save")
	//@ApiOperationSupport(order = 4)
	//@ApiOperation(value = "新增", notes = "传入paymentOrderRecord")
	public R save(@Valid @RequestBody PaymentOrderRecord paymentOrderRecord) {
		return R.status(paymentOrderRecordService.save(paymentOrderRecord));
	}

	/**
	 * 修改 支付明细表
	 */
	@PostMapping("/update")
	//@ApiOperationSupport(order = 5)
	//@ApiOperation(value = "修改", notes = "传入paymentOrderRecord")
	public R update(@Valid @RequestBody PaymentOrderRecord paymentOrderRecord) {
		return R.status(paymentOrderRecordService.updateById(paymentOrderRecord));
	}

	/**
	 * 新增或修改 支付明细表
	 */
	@PostMapping("/submit")
	//@ApiOperationSupport(order = 6)
	//@ApiOperation(value = "新增或修改", notes = "传入paymentOrderRecord")
	public R submit(@Valid @RequestBody PaymentOrderRecord paymentOrderRecord) {
		return R.status(paymentOrderRecordService.saveOrUpdate(paymentOrderRecord));
	}

	
	/**
	 * 删除 支付明细表
	 */
	@PostMapping("/remove")
	//@ApiOperationSupport(order = 8)
	//@ApiOperation(value = "删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(paymentOrderRecordService.removeByIds(Func.toLongList(ids)));
	}

	
}
