/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.app.entity;

import java.math.BigDecimal;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import java.time.LocalDateTime;
import java.io.Serializable;
import lombok.Data;
import lombok.EqualsAndHashCode;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

/**
 * 支付明细表实体类
 *
 * @author BladeX
 * @since 2022-10-14
 */
@Data
@TableName("tb_payment_order_record")
@ApiModel(value = "PaymentOrderRecord对象", description = "支付明细表")
public class PaymentOrderRecord implements Serializable {

	private static final long serialVersionUID = 1L;

	/**
	* 主键
	*/
		@ApiModelProperty(value = "主键")
		@TableId(value = "id", type = IdType.AUTO)
	private Long id;
	/**
	* 订单id
	*/
		@ApiModelProperty(value = "订单id")
		private Long orderId;
	/**
	* 支付订单号（第三方返回的订单号）
	*/
		@ApiModelProperty(value = "支付订单号（第三方返回的订单号）")
		private String payOrderCode;
	/**
	* 支付渠道
	*/
		@ApiModelProperty(value = "支付渠道")
		private String transactionChannel;
	/**
	* 支付方式
	*/
		@ApiModelProperty(value = "支付方式")
		private String transactionMethod;
	/**
	* 支付类型（支付宝，微信）
	*/
		@ApiModelProperty(value = "支付类型（支付宝，微信）")
		private String payMode;
	/**
	* 流水号
	*/
		@ApiModelProperty(value = "流水号")
		private String tradeNo;
	/**
	* 支付金额
	*/
		@ApiModelProperty(value = "支付金额")
		private Long payMoney;
	/**
	* 付款方账户id
	*/
		@ApiModelProperty(value = "付款方账户id")
		private Long payerAccountId;
	/**
	* 付款方户头号
	*/
		@ApiModelProperty(value = "付款方户头号")
		private String payerAccountNum;
	/**
	* 收款方账户id
	*/
		@ApiModelProperty(value = "收款方账户id")
		private Long payeeAccountId;
	/**
	* 收款方户头号/商户号
	*/
		@ApiModelProperty(value = "收款方户头号/商户号")
		private String payeeAccountNum;
	/**
	* 支付状态 0:新订单,1:支付中,2:已支付,-1:取消,-2:失败
	*/
		@ApiModelProperty(value = "支付状态 0:新订单,1:支付中,2:已支付,-1:取消,-2:失败")
		private Integer payStatus;
	/**
	* 支付开始时间
	*/
		@ApiModelProperty(value = "支付开始时间")
		private LocalDateTime timeStartPayment;
	/**
	* 支付完成时间
	*/
		@ApiModelProperty(value = "支付完成时间")
		private LocalDateTime timePaid;
	/**
	* 支付取消时间
	*/
		@ApiModelProperty(value = "支付取消时间")
		private LocalDateTime timeCancelled;
	/**
	* 创建人
	*/
		@ApiModelProperty(value = "创建人")
		private Long createUser;
	/**
	* 创建部门
	*/
		@ApiModelProperty(value = "创建部门")
		private Long createDept;
	/**
	* 创建时间
	*/
		@ApiModelProperty(value = "创建时间")
		private LocalDateTime createTime;
	/**
	* 修改人
	*/
		@ApiModelProperty(value = "修改人")
		private Long updateUser;
	/**
	* 修改时间
	*/
		@ApiModelProperty(value = "修改时间")
		private LocalDateTime updateTime;
	/**
	* 状态
	*/
		@ApiModelProperty(value = "状态")
		private Integer status;
	/**
	* 是否启用0：启用，1：不启用
	*/
		@ApiModelProperty(value = "是否启用0：启用，1：不启用")
		private Integer isDeleted;


}
