/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.desk.entity;

import com.baomidou.mybatisplus.annotation.TableName;
import java.time.LocalDateTime;
import java.io.Serializable;
import lombok.Data;
import lombok.EqualsAndHashCode;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

/**
 * 用户信息表实体类
 *
 * @author BladeX
 * @since 2022-05-25
 */
@Data
@TableName("tb_user")
@ApiModel(value = "PlatformUser对象", description = "用户信息表")
public class PlatformUser implements Serializable {

	private static final long serialVersionUID = 1L;

	/**
	* 用户id
	*/
		@ApiModelProperty(value = "用户id")
		private Long id;

		private String sysUserId;
	/**
	* 手机号
	*/
		@ApiModelProperty(value = "手机号")
		private String phone;
	/**
	* 用户密码
	*/
		@ApiModelProperty(value = "用户密码")
		private String userPassword;
	/**
	* 浙牧通ID
	*/
		@ApiModelProperty(value = "浙牧通ID")
		private Long zmtId;
	/**
	* 浙牧通用户ID
	*/
		@ApiModelProperty(value = "浙牧通用户ID")
		private Long zmtUserId;
	/**
	* 畜禽养殖代码
	*/
		@ApiModelProperty(value = "畜禽养殖代码")
		private String breedCode;
	/**
	* 浙牧通用户名
	*/
		@ApiModelProperty(value = "浙牧通用户名")
		private String zmtUserName;
	/**
	* 浙牧通主体名称
	*/
		@ApiModelProperty(value = "浙牧通主体名称")
		private String zmtName;
	/**
	* 浙牧通调运主体类型 0-无 1 养殖场 2 屠宰场 3 其他调运主体
	*/
		@ApiModelProperty(value = "浙牧通调运主体类型 0-无 1 养殖场 2 屠宰场 3 其他调运主体")
		private Integer zmtType;
	/**
	* 类型 1-屠宰场 2-养殖户 3-司机 4-经纪人 5-屠宰户
	*/
		@ApiModelProperty(value = "类型 1-屠宰场 2-养殖户 3-司机 4-经纪人 5-屠宰户")
		private Integer type;
	/**
	* 状态 0-初始 1-启用 -1-禁用
	*/
		@ApiModelProperty(value = "状态 0-初始 1-启用 -1-禁用")
		private Integer state;
	/**
	* 联系人名称
	*/
		@ApiModelProperty(value = "联系人名称")
		private String businessLinkMan;
	/**
	* 用户名称(姓名/企业名称)
	*/
		@ApiModelProperty(value = "用户名称(姓名/企业名称)")
		private String userName;
	/**
	* 省
	*/
		@ApiModelProperty(value = "省")
		private Long provId;
	/**
	* 省名称
	*/
		@ApiModelProperty(value = "省名称")
		private String provName;
	/**
	* 市
	*/
		@ApiModelProperty(value = "市")
		private Long cityId;
	/**
	* 市名称
	*/
		@ApiModelProperty(value = "市名称")
		private String cityName;
	/**
	* 区
	*/
		@ApiModelProperty(value = "区")
		private Long areaId;
	/**
	* 区名称
	*/
		@ApiModelProperty(value = "区名称")
		private String areaName;
	/**
	* 用户地址
	*/
		@ApiModelProperty(value = "用户地址")
		private String address;
	/**
	* 行业类型
	*/
		@ApiModelProperty(value = "行业类型")
		private String businessIndustryCategrory;
	/**
	* 法定代表人
	*/
		@ApiModelProperty(value = "法定代表人")
		private String businessLegalPerson;
	/**
	* 注册资金
	*/
		@ApiModelProperty(value = "注册资金")
		private String businessRegisterMoney;
	/**
	* 注册时间
	*/
		@ApiModelProperty(value = "注册时间")
		private LocalDateTime businessRegisterDate;
	/**
	* 营业期限
	*/
		@ApiModelProperty(value = "营业期限")
		private LocalDateTime businessTerm;
	/**
	* 营业范围
	*/
		@ApiModelProperty(value = "营业范围")
		private String businessScope;
	/**
	* 唯一认证编码(身份证/统一社会信用编码)
	*/
		@ApiModelProperty(value = "唯一认证编码(身份证/统一社会信用编码)")
		private String creditCode;
	/**
	* 营业执照url
	*/
		@ApiModelProperty(value = "营业执照url")
		private String businessUrl;
	/**
	* 身份证正面
	*/
		@ApiModelProperty(value = "身份证正面")
		private String idCardFrontUrl;
	/**
	* 身份证背面
	*/
		@ApiModelProperty(value = "身份证背面")
		private String idCardBehindUrl;
	/**
	* 动物防疫条件合格证编码
	*/
		@ApiModelProperty(value = "动物防疫条件合格证编码")
		private String livestockPreventionPassCode;
	/**
	* 动物防疫条件合格证
	*/
		@ApiModelProperty(value = "动物防疫条件合格证")
		private String livestockPreventionPassUrl;
	/**
	* 生猪定点屠宰许可证编码
	*/
		@ApiModelProperty(value = "生猪定点屠宰许可证编码")
		private String livestockSlaughterPermitCode;
	/**
	* 生猪定点屠宰许可证
	*/
		@ApiModelProperty(value = "生猪定点屠宰许可证")
		private String livestockSlaughterPermitUrl;
	/**
	* 排污许可证编码
	*/
		@ApiModelProperty(value = "排污许可证编码")
		private String pollutantDischargePermitCode;
	/**
	* 排污许可证行业类别
	*/
		@ApiModelProperty(value = "排污许可证行业类别")
		private String pollutantDischargePermitIndustry;
	/**
	* 排污许可证有效期
	*/
		@ApiModelProperty(value = "排污许可证有效期")
		private LocalDateTime pollutantDischargePermitDate;
	/**
	* 排污许可证
	*/
		@ApiModelProperty(value = "排污许可证")
		private String pollutantDischargePermitLicenseUrl;
	/**
	* 无疫情证明到期日期
	*/
		@ApiModelProperty(value = "无疫情证明到期日期")
		private LocalDateTime noEpidemicProofPassDate;
	/**
	* 三年无疫情证明
	*/
		@ApiModelProperty(value = "三年无疫情证明")
		private String noEpidemicProofPassUrl;
	/**
	* 驾驶证编码
	*/
		@ApiModelProperty(value = "驾驶证编码")
		private String driverLicenceCode;
	/**
	* 驾驶证
	*/
		@ApiModelProperty(value = "驾驶证")
		private String driverLicenceUrl;
	/**
	* 准驾车型
	*/
		@ApiModelProperty(value = "准驾车型")
		private String driverCarType;
	/**
	* 驾驶证发放单位
	*/
		@ApiModelProperty(value = "驾驶证发放单位")
		private String driverIssuing;
	/**
	* 驾驶证初次发放日期
	*/
		@ApiModelProperty(value = "驾驶证初次发放日期")
		private LocalDateTime driverFirstIssueDate;
	/**
	* 驾驶证到期日期
	*/
		@ApiModelProperty(value = "驾驶证到期日期")
		private LocalDateTime driverLicenceExpiryDate;
	/**
	* 行驶证编码
	*/
		@ApiModelProperty(value = "行驶证编码")
		private String vehicleLicenseCode;
	/**
	* 行驶证
	*/
		@ApiModelProperty(value = "行驶证")
		private String vehicleLicenseUrl;
	/**
	* 道路运输许可证编码
	*/
		@ApiModelProperty(value = "道路运输许可证编码")
		private String transportPermitCode;
	/**
	* 道路运输许可证
	*/
		@ApiModelProperty(value = "道路运输许可证")
		private String transportPermitUrl;
	/**
	* 挂靠的屠宰场id
	*/
		@ApiModelProperty(value = "挂靠的屠宰场id")
		private Long slaughterHouseParentId;
	/**
	* 挂靠的屠宰场名称
	*/
		@ApiModelProperty(value = "挂靠的屠宰场名称")
		private String slaughterHouseParentName;
	/**
	* 待宰户类型 1-自然人 2-个体户 3-企业主
	*/
		@ApiModelProperty(value = "待宰户类型 1-自然人 2-个体户 3-企业主")
		private Integer slaughterHouseType;
	/**
	* 版本号，用于乐观锁处理
	*/
		@ApiModelProperty(value = "版本号，用于乐观锁处理")
		private Integer version;
	/**
	* 逻辑删除标识。1：此记录已经删除
	*/
		@ApiModelProperty(value = "逻辑删除标识。1：此记录已经删除")
		private Integer isDeleted;
	/**
	* 创建时间
	*/
		@ApiModelProperty(value = "创建时间")
		private LocalDateTime createdTime;
	/**
	* 创建者
	*/
		@ApiModelProperty(value = "创建者")
		private String creator;
	/**
	* 修改时间
	*/
		@ApiModelProperty(value = "修改时间")
		private LocalDateTime modifiedTime;
	/**
	* 修改者
	*/
		@ApiModelProperty(value = "修改者")
		private String modifier;


}
